package com.ruoyi.project.pssr.mapper;

import java.util.List;
import com.ruoyi.framework.aspectj.lang.annotation.DataScope;
import com.ruoyi.project.pssr.domain.TPssrMotor;

/**
 * 电机设备Mapper接口
 *
 * @author ssy
 * @date 2024-09-18
 */
public interface TPssrMotorMapper
{
    /**
     * 查询电机设备
     *
     * @param id 电机设备ID
     * @return 电机设备
     */
    public TPssrMotor selectTPssrMotorById(Long id);
    public int selectNoConfirmCount(Long subId);

    /**
     * 查询电机设备列表
     *
     * @param tPssrMotor 电机设备
     * @return 电机设备集合
     */
    
    public List<TPssrMotor> selectTPssrMotorList(TPssrMotor tPssrMotor);

    /**
     * 新增电机设备
     *
     * @param tPssrMotor 电机设备
     * @return 结果
     */
    public int insertTPssrMotor(TPssrMotor tPssrMotor);

    /**
     * 修改电机设备
     *
     * @param tPssrMotor 电机设备
     * @return 结果
     */
    public int updateTPssrMotor(TPssrMotor tPssrMotor);
    public int updateTPssrMotorByIds(TPssrMotor tPssrMotor);

    /**
     * 删除电机设备
     *
     * @param id 电机设备ID
     * @return 结果
     */
    public int deleteTPssrMotorById(Long id);

    /**
     * 批量删除电机设备
     *
     * @param ids 需要删除的数据ID
     * @return 结果
     */
    public int deleteTPssrMotorByIds(Long[] ids);
}
